///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                         Zonerings Settings Control                        //
//                             John Biddiscombe                             //
//                           j.biddiscombe@rl.ac.uk                          //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
unit Zone_settings;

interface

uses
	{ Borland }
	Windows, Forms, Classes, SysUtils, Dialogs,
	{ Mine }
	Thinkbox, PalUnit, Str_util, My_LZH, UBag;

type
	Zone_error = (
		no_reg_write, no_reg_read, directory_invalid,
		settings_not_found, settings_corrupt,
		INI_file_not_found, palettes_not_found,
		mode_not_found,  mode_change_not_allowed, mode_change_restart, mode_change_failed
	);

const
	empty_set : set of Zone_error = [];
	
type
	Zone_setting = class(TObject)
		control_strings	: TStringList;
		constructor Create;
		destructor	Destroy; override;
		function		ReadText(source:TMemoryStream) : boolean;
		procedure		WriteText(dest:TMemoryStream);
		function		name	: string;
		function		screensave	: boolean;
		procedure		setname(newname:string);
		procedure		setscreensave(newval:boolean);
	end;

type
	Zone_settings_file = class(TObject)
		file_name				: string;
		Modified				: boolean;
		Settings				: TList;
		memory					: TMemoryStream;
		compressed_data	: My_LZH_Compressor;
		constructor	Create;
		destructor	Destroy; override;
		function		Open_File(filename:string; showthoughts:boolean) : boolean;
		procedure		Close_File;
		procedure		Clear;
		procedure		Read_settings_data;
		procedure		Save_settings_data;
		function		Entries	: integer;
		function		Entry(index:integer) : Zone_setting;
		function		Random_Setting(tagged_only:boolean) : integer;
		procedure		Delete(index:integer);
		procedure		Add(setting:Zone_setting);
		procedure		Overwrite(index:integer; setting:Zone_setting);
		procedure		Set_setting_name(index:integer; name:string);
		procedure		Set_setting_screensave(index:integer; val:boolean);
	end;

	Zone_Options = class(TObject)
		Zone_Directory	: String;
		Cycling_mode		: boolean;
		Startup_cycling	: boolean;
		Cycle_tagged		: boolean;
		Min_time				: Integer;
		Max_time				: Integer;
		settings_name		: String;
		Prompt_change		: boolean;
		Prompt_force		: boolean;
		// we'll use a global error flag
		error_flag			: set of Zone_error;
		error_string		: String;
		constructor	Create;
		destructor	destroy; override;
		procedure		read_ini;
		procedure		write_ini;
	end;

var
	ZOptions	: Zone_Options;

const
	eol : string = #13#10;

implementation

///////////////////////////////////////////////////////////////////////////////
//	Zone_setting methods
///////////////////////////////////////////////////////////////////////////////
constructor Zone_setting.Create;
begin
	control_strings	:= TStringList.Create;
end;

destructor Zone_setting.Destroy;
begin
	control_strings.Free;
end;

function Zone_setting.ReadText(source:TMemoryStream) : boolean;
var line	: string;
		done	: boolean;
begin
	control_strings.Clear;
	result	:= true;
	done		:= false;
	line		:= read_line(source);
	if line[1] = '{' then begin
		line[1]:=' '; line:=Trim(line);
		if line<>'' then control_strings.Add(line);
	end else result	:= false;
	if result then repeat
		line	:= read_line(source);
		if line='' then begin end
		else if line[1] = '}' then done	:= true
		else control_strings.Add(line);
	until done or (not result);
end;

procedure Zone_setting.WriteText(dest:TMemoryStream);
var lp1	: integer;
begin
	WriteString(dest,'{ ' + control_strings[0] );
	for lp1	:= 1 to control_strings.count-1 do WriteString(dest,control_strings[lp1]);
	WriteString(dest,'}');
end;

function Zone_setting.name	: string;
begin
	result	:= control_strings[0];
end;

function Zone_setting.screensave	: boolean;
var lp1	: integer;
begin
	result	:= false;
	for lp1:=0 to control_strings.count-1 do begin
		if pos('Screensaver',control_strings[lp1])<>0 then begin
			if pos('TRUE',control_strings[lp1])<>0 then begin result:=true; exit; end
			else result:=false;
		end;
	end;
end;

procedure Zone_setting.setname(newname:string);
begin
	control_strings[0]	:= newname;
end;

procedure Zone_setting.setscreensave(newval:boolean);
var lp1	: integer;
begin
	for lp1:=0 to control_strings.count-1 do begin
		if pos('Screensaver',control_strings[lp1])<>0 then begin
			control_strings[lp1]	:= Format('%20s',['Screensaver'])+#9+BoolToStr(newval);
		exit;
		end;
	end;
	control_strings.Add(Format('%20s',['Screensaver'])+#9+BoolToStr(newval));
end;

///////////////////////////////////////////////////////////////////////////////
//	Zone_settings_file methods
///////////////////////////////////////////////////////////////////////////////
constructor Zone_settings_file.Create;
begin
	Settings				:= TList.create;
	compressed_data	:= My_LZH_Compressor.Create;
	memory					:= compressed_data.i_s;
	Modified				:= false;
end;

destructor Zone_settings_file.Destroy;
var lp1	: integer;
begin
	if Modified then Save_settings_data;
	for lp1:=0 to Entries-1 do Entry(lp1).Free;
	Settings.Free;
	compressed_data.Free;
	inherited destroy;
end;

function Zone_settings_file.Open_File(filename:string; showthoughts:boolean) : boolean;
var Use_Compression	: boolean;
begin
	Clear;
	file_name	:= filename;
	if ExtractFileExt(file_name)='.cmp' then Use_Compression	:= true
	else                                     Use_Compression	:= false;
	result	:= true;
	if FileExists(file_name) then begin
		if (Use_Compression) then begin
			if showthoughts then thinking(nil,'Decompressing Zonerings settings data');
			compressed_data.o_s.LoadFromFile(file_name);
			compressed_data.compression(false);
		end
		else compressed_data.i_s.LoadFromFile(file_name);
	end
	else begin
		ZOptions.error_flag		:= ZOptions.error_flag + [settings_not_found];
		ZOptions.error_string	:= ZOptions.error_string + 
			'Zonerings could not find the settings file ' + file_name + eol + eol;
		result	:= false;
	end;
	if result then Read_settings_data;
	Modified	:= false;
end;

procedure Zone_settings_file.Close_File;
begin
	if Modified then Save_settings_data;
	Clear;
end;

procedure Zone_settings_file.Clear;
var lp1	: integer;
begin
	for lp1:=0 to Entries-1 do Entry(lp1).Free;
	Settings.Clear;
	Memory.Clear;
end;

procedure Zone_settings_file.Read_settings_data;
var entry	: Zone_setting;
begin
	memory.position	:= 0;
	repeat
		entry	:= Zone_setting.Create;
		if not entry.ReadText(memory) then begin
			ZOptions.error_flag		:= ZOptions.error_flag + [settings_corrupt];;
			ZOptions.error_string	:= ZOptions.error_string +
				'The Zonerings settings file appears to have been corrupted' + eol + eol;
			entry.Free;
			Clear;
			break;
		end;
		Settings.Add(entry);
	until memory.position = memory.size;
	memory.Clear;
end;

procedure Zone_settings_file.Save_settings_data;
var lp1							: integer;
		Use_Compression	: boolean;
begin
	memory.Clear;
	for lp1:=0 to Entries-1 do Entry(lp1).WriteText(memory);
	if ExtractFileExt(file_name)='.cmp' then Use_Compression	:= true
	else                                     Use_Compression	:= false;
	if Use_Compression then begin
		compressed_data.progressbar			:= progress_bar;
		compressed_data.progressbar.max	:= trunc(compressed_data.i_s.Size/1024);
		thinking_bar(nil,'Compressing Zonerings settings data');
		compressed_data.compression(true);
		stopped_thinking;
		compressed_data.o_s.SaveToFile(file_name);
	end
	else compressed_data.i_s.SaveToFile(file_name);
	Modified	:= false;
end;

function Zone_settings_file.Entries	: integer;
begin
	result	:= Settings.Count;
end;

function Zone_settings_file.Entry(index:integer) : Zone_setting;
begin
	result	:= Settings[index];
end;

function Zone_settings_file.Random_Setting(tagged_only:boolean) : integer;
var lp1	: integer;
begin
	result	:= -1;
	if Entries=0 then exit;
	if tagged_only then begin
		// keep selecting random settings until one with flag set
		for lp1:=0 to 5000 do begin
			result	:= random(Settings.count);
			if Entry(result).Screensave then exit;
		end;
	end;
	// if we didn't find a tagged one, then just pick any old one
	result	:= random(Settings.count);
end;

procedure Zone_settings_file.Delete(index:integer);
var zs	: Zone_setting;
begin
	zs	:= Entry(index);
	zs.Free;
	Settings.Remove(zs);
end;

procedure Zone_settings_file.Add(setting:Zone_setting);
begin
	Settings.Add(setting);
	Modified	:= true;
end;

procedure Zone_settings_file.Overwrite(index:integer; setting:Zone_setting);
begin
	Entry(index).Free;
	Settings[index]	:= setting;
	Modified				:= true;
end;

procedure Zone_settings_file.Set_setting_name(index:integer; name:string);
begin
	Entry(index).setname(name);
	Modified	:= true;
end;

procedure Zone_settings_file.Set_setting_screensave(index:integer; val:boolean);
begin
	Entry(index).setscreensave(val);
	Modified	:= true;
end;

///////////////////////////////////////////////////////////////////////////////
//	Zone_Options
///////////////////////////////////////////////////////////////////////////////
constructor Zone_Options.Create;
var res,disp : integer;
	skey		 : HKEY;
	buff		 : array[0..1023] of char;
{$IFDEF SCREENSAVER }
	d_type,d_len : integer;
{$ENDIF }

begin
	buff[0] := #0;
	Zone_Directory := '';
	error_string   := '';
	{$IFNDEF SCREENSAVER }
		Zone_Directory := ExtractFilePath(Application.ExeName);
		res	:= RegCreateKeyEx(HKEY_CURRENT_USER,'Software\Zonerings', 0,nil,
			REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,nil,skey,@disp);
		if res = ERROR_SUCCESS then begin
			res	:= RegSetValueEx(skey,'Directory',0,REG_SZ,@Zone_Directory[1],Length(Zone_Directory));
			RegCloseKey(skey);
		end;
		if res <> ERROR_SUCCESS then begin
			error_flag		:= error_flag + [no_reg_write];
			error_string	:= error_string +
				'Failed to write registry key :' + eol +
				'HKEY_CURRENT_USER\Software\Zonerings' + eol + eol;
			end;
	{$ELSE }
		res := RegOpenKeyEx(HKEY_CURRENT_USER,'Software\Zonerings', 0,
			KEY_ALL_ACCESS,skey);
		d_len	:= 1024;
		if res = ERROR_SUCCESS then begin
			res	:= RegQueryValueEx(skey,'Directory',nil,@d_type,@buff,@d_len);
			Zone_Directory	:= StrPas(buff);
			RegCloseKey(skey);
		end;
		if ((res <> ERROR_SUCCESS) or (Zone_Directory='')) then begin
			error_flag		:= error_flag + [no_reg_read];;
			error_string	:= error_string +
				'Failed to find registry key :' + eol +
				'HKEY_CURRENT_USER\Software\Zonerings' + eol +
				'Please run the Zonerings "exe" file : ' +
				'This will write the directory of the Zonerings data files to the registry.' + eol + eol;
			Zone_Directory	:= '';
			exit;
		end;
	{$ENDIF }
	if not DirectoryExists(Zone_Directory) then begin
		error_flag		:= error_flag + [directory_invalid];;
		error_string	:= error_string +
			'The Zonerings directory ' + Zone_Directory + ' is invalid' + eol +
			'Please run the Zonerings "exe" file : ' +
			'This will write the directory of the Zonerings data files to the registry.' + eol + eol;
	end;
	read_ini;
end;

destructor Zone_Options.Destroy;
begin
	write_ini;
	inherited destroy;
end;

procedure Zone_Options.read_ini;
var Zone_INI	: TBag;
begin
	Zone_INI	:= TBag.create(nil);
	with Zone_INI do begin
		INIFile	:= Zone_Directory + 'Zone_Rings_32_Config.ini';
{
		if not FileExists(INIFile) then begin
		  error_flag   := error_flag + [INI_file_not_found];
			error_string := error_string + 'Zonerings couldn''t find the  INI file '+INIfile+eol;
		end;
}
		INISect	:= 'Zone_start';
		ReadIni;
		Startup_cycling	:= GetBoolean('Start_auto_cycling',True);
		Cycling_mode		:= Startup_cycling;
		Cycle_tagged		:= GetBoolean('Tagged_settings_only',True);
		Min_time				:= GetInteger('Min_time',30);
		Max_time				:= GetInteger('Max_time',60);
		Prompt_change		:= GetBoolean('Prompt_screen_change',True);
		Prompt_force		:= GetBoolean('Prompt_force_change',True);
		settings_name		:= GetString ('Settings_file','Zone_Rings_32_Settings.cmp');
	end;
	Zone_Ini.Free;
end;

procedure Zone_Options.write_ini;
var Zone_INI	: TBag;
begin
	Zone_INI	:= TBag.create(nil);
	with Zone_INI do begin
		INIFile	:= Zone_Directory + 'Zone_Rings_32_Config.ini';
		INISect	:= 'Zone_start';
		SetBoolean('Start_auto_cycling',Startup_cycling);
		SetBoolean('Tagged_settings_only',Cycle_tagged);
		SetInteger('Min_time',Min_time);
		SetInteger('Max_time',Max_time);
		SetBoolean('Prompt_screen_change',Prompt_change);
		SetBoolean('Prompt_force_change',Prompt_force);
		SetString ('Settings_file',settings_name);
		WriteIni;
	end;
	Zone_Ini.Free;
end;

initialization
	randomize;
end.
